package techreborn.tiles.fusionReactor;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.ITextComponent;
import reborncore.api.power.EnumPowerTier;
import reborncore.api.tile.IInventoryProvider;
import reborncore.common.powerSystem.TilePowerAcceptor;
import reborncore.common.util.Inventory;
import reborncore.common.util.ItemUtils;
import techreborn.api.reactor.FusionReactorRecipe;
import techreborn.api.reactor.FusionReactorRecipeHelper;
import techreborn.init.ModBlocks;

public class TileEntityFusionController extends TilePowerAcceptor implements IInventoryProvider
{

	public Inventory inventory = new Inventory(3, "TileEntityFusionController", 64, this);

	// 0= no coils, 1 = coils
	public int coilStatus = 0;
	public int crafingTickTime = 0;
	public int finalTickTime = 0;
	public int neededPower = 0;
	int topStackSlot = 0;
	int bottomStackSlot = 1;
	int outputStackSlot = 2;
	FusionReactorRecipe currentRecipe = null;
	boolean hasStartedCrafting = false;

	public TileEntityFusionController()
	{
		super(4);
	}

	@Override
	public double getMaxPower()
	{
		return 100000000;
	}

	@Override
	public boolean canAcceptEnergy(EnumFacing direction)
	{
		return !(direction == EnumFacing.DOWN || direction == EnumFacing.UP);
	}

	@Override
	public boolean canProvideEnergy(EnumFacing direction)
	{
		return direction == EnumFacing.DOWN || direction == EnumFacing.UP;
	}

	@Override
	public double getMaxOutput()
	{
		if (!hasStartedCrafting)
		{
			return 0;
		}
		return 1000000;
	}

	@Override
	public double getMaxInput()
	{
		if (hasStartedCrafting)
		{
			return 0;
		}
		return 8192;
	}

	@Override
	public EnumPowerTier getTier()
	{
		return EnumPowerTier.EXTREME;
	}

	@Override
	public void func_145839_a(NBTTagCompound tagCompound)
	{
		super.func_145839_a(tagCompound);
		crafingTickTime = tagCompound.func_74762_e("crafingTickTime");
		finalTickTime = tagCompound.func_74762_e("finalTickTime");
		neededPower = tagCompound.func_74762_e("neededPower");
		hasStartedCrafting = tagCompound.func_74767_n("hasStartedCrafting");
	}

	@Override
	public NBTTagCompound func_189515_b(NBTTagCompound tagCompound)
	{
		super.func_189515_b(tagCompound);

		if (crafingTickTime == -1)
		{
			crafingTickTime = 0;
		}
		if (finalTickTime == -1)
		{
			finalTickTime = 0;
		}
		if (neededPower == -1)
		{
			neededPower = 0;
		}
		tagCompound.func_74768_a("crafingTickTime", crafingTickTime);
		tagCompound.func_74768_a("finalTickTime", finalTickTime);
		tagCompound.func_74768_a("neededPower", neededPower);
		tagCompound.func_74757_a("hasStartedCrafting", hasStartedCrafting);
		return tagCompound;
	}


	public boolean checkCoils()
	{
		if ((isCoil(this.func_174877_v().func_177958_n() + 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 1))
				&& (isCoil(this.func_174877_v().func_177958_n() + 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p()))
				&& (isCoil(this.func_174877_v().func_177958_n() + 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 1))
				&& (isCoil(this.func_174877_v().func_177958_n() - 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 1))
				&& (isCoil(this.func_174877_v().func_177958_n() - 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p()))
				&& (isCoil(this.func_174877_v().func_177958_n() - 3, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 1))
				&& (isCoil(this.func_174877_v().func_177958_n() + 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 2))
				&& (isCoil(this.func_174877_v().func_177958_n() + 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 1))
				&& (isCoil(this.func_174877_v().func_177958_n() + 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 1))
				&& (isCoil(this.func_174877_v().func_177958_n() + 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 2))
				&& (isCoil(this.func_174877_v().func_177958_n() - 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 2))
				&& (isCoil(this.func_174877_v().func_177958_n() - 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 1))
				&& (isCoil(this.func_174877_v().func_177958_n() - 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 1))
				&& (isCoil(this.func_174877_v().func_177958_n() - 2, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 2))
				&& (isCoil(this.func_174877_v().func_177958_n() + 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 3))
				&& (isCoil(this.func_174877_v().func_177958_n() + 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 2))
				&& (isCoil(this.func_174877_v().func_177958_n() + 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 2))
				&& (isCoil(this.func_174877_v().func_177958_n() + 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 3))
				&& (isCoil(this.func_174877_v().func_177958_n() - 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 3))
				&& (isCoil(this.func_174877_v().func_177958_n() - 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 2))
				&& (isCoil(this.func_174877_v().func_177958_n() - 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 2))
				&& (isCoil(this.func_174877_v().func_177958_n() - 1, this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 3))
				&& (isCoil(this.func_174877_v().func_177958_n(), this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() + 3))
				&& (isCoil(this.func_174877_v().func_177958_n(), this.func_174877_v().func_177956_o(), this.func_174877_v().func_177952_p() - 3)))
		{
			coilStatus = 1;
			return true;
		}
		coilStatus = 0;
		return false;
	}

	private boolean isCoil(int x, int y, int z)
	{
		return field_145850_b.func_180495_p(new BlockPos(x, y, z)).func_177230_c() == ModBlocks.FusionCoil;
	}

	@Override
	public void updateEntity()
	{
		super.updateEntity();
		// TODO improve this code a lot

		if (field_145850_b.func_82737_E() % 20 == 0)
		{
			checkCoils();
		}

		if (!field_145850_b.field_72995_K)
		{
			if (coilStatus == 1)
			{
				if (currentRecipe == null)
				{
					if (inventory.hasChanged || crafingTickTime != 0)
					{
						for (FusionReactorRecipe reactorRecipe : FusionReactorRecipeHelper.reactorRecipes)
						{
							if (ItemUtils.isItemEqual(func_70301_a(topStackSlot), reactorRecipe.getTopInput(), true,
									true, true))
							{
								if (reactorRecipe.getBottomInput() != null)
								{
									if (!ItemUtils.isItemEqual(func_70301_a(bottomStackSlot),
											reactorRecipe.getBottomInput(), true, true, true))
									{
										break;
									}
								}
								if (canFitStack(reactorRecipe.getOutput(), outputStackSlot, true))
								{
									currentRecipe = reactorRecipe;
									if (crafingTickTime != 0)
									{
										finalTickTime = currentRecipe.getTickTime();
										neededPower = (int) currentRecipe.getStartEU();
									}
									hasStartedCrafting = false;
									crafingTickTime = 0;
									finalTickTime = currentRecipe.getTickTime();
									neededPower = (int) currentRecipe.getStartEU();
									break;
								}
							}
						}
					}
				} else
				{
					if (inventory.hasChanged)
					{
						if (!validateRecipe())
						{
							resetCrafter();
							return;
						}
					}
					if (!hasStartedCrafting)
					{
						if (canUseEnergy(currentRecipe.getStartEU() + 64))
						{
							useEnergy(currentRecipe.getStartEU());
							hasStartedCrafting = true;
						}
					} else
					{
						if (crafingTickTime < currentRecipe.getTickTime())
						{
							if (currentRecipe.getEuTick() > 0)
							{ // Power gen
								addEnergy(currentRecipe.getEuTick()); // Waste
																		// power
																		// if it
																		// has
																		// no
																		// where
																		// to go
								crafingTickTime++;
							} else
							{ // Power user
								if (canUseEnergy(currentRecipe.getEuTick() * -1))
								{
									setEnergy(getEnergy() - (currentRecipe.getEuTick() * -1));
									crafingTickTime++;
								}
							}
						} else
						{
							if (canFitStack(currentRecipe.getOutput(), outputStackSlot, true))
							{
								if (func_70301_a(outputStackSlot) == null)
								{
									func_70299_a(outputStackSlot, currentRecipe.getOutput().func_77946_l());
								} else
								{
									func_70298_a(outputStackSlot, -currentRecipe.getOutput().field_77994_a);
								}
								func_70298_a(topStackSlot, currentRecipe.getTopInput().field_77994_a);
								if (currentRecipe.getBottomInput() != null)
								{
									func_70298_a(bottomStackSlot, currentRecipe.getBottomInput().field_77994_a);
								}
								resetCrafter();
							}
						}
					}
				}
			} else
			{
				if (currentRecipe != null)
				{
					resetCrafter();
				}
			}
		}

		if (inventory.hasChanged)
		{
			inventory.hasChanged = false;
		}
	}

	private boolean validateRecipe()
	{
		if (ItemUtils.isItemEqual(func_70301_a(topStackSlot), currentRecipe.getTopInput(), true, true, true))
		{
			if (currentRecipe.getBottomInput() != null)
			{
				if (!ItemUtils.isItemEqual(func_70301_a(bottomStackSlot), currentRecipe.getBottomInput(), true, true,
						true))
				{
					return false;
				}
			}
			if (canFitStack(currentRecipe.getOutput(), outputStackSlot, true))
			{
				return true;
			}
		}
		return false;
	}

	private void resetCrafter()
	{
		currentRecipe = null;
		crafingTickTime = -1;
		finalTickTime = -1;
		neededPower = -1;
		hasStartedCrafting = false;
	}

	public boolean canFitStack(ItemStack stack, int slot, boolean oreDic)
	{// Checks to see if it can fit the stack
		if (stack == null)
		{
			return true;
		}
		if (inventory.func_70301_a(slot) == null)
		{
			return true;
		}
		if (ItemUtils.isItemEqual(inventory.func_70301_a(slot), stack, true, true, oreDic))
		{
			if (stack.field_77994_a + inventory.func_70301_a(slot).field_77994_a <= stack.func_77976_d())
			{
				return true;
			}
		}
		return false;
	}

	@Override
	public String func_70005_c_()
	{
		return null;
	}

	@Override
	public boolean func_145818_k_()
	{
		return false;
	}

	@Override
	public ITextComponent func_145748_c_()
	{
		return null;
	}

	@Override
	public Inventory getInventory() {
		return inventory;
	}
}
